@@ -48,9 +48,13 @@ DBusMessage *__connman_error_invalid_property(DBusMessage *msg);
 
 int __connman_selftest(void);
 
+#include <connman/types.h>
+
 int __connman_manager_init(DBusConnection *conn, gboolean compat);
 void __connman_manager_cleanup(void);
 
+connman_bool_t __connman_manager_get_offlinemode(void);
+
 int __connman_agent_init(DBusConnection *conn);
 void __connman_agent_cleanup(void);
 
@@ -124,12 +128,15 @@ int __connman_resolver_selftest(void);
 int __connman_storage_init(void);
 void __connman_storage_cleanup(void);
 
+GKeyFile *__connman_storage_open(void);
+void __connman_storage_close(GKeyFile *keyfile, gboolean save);
+
+int __connman_storage_load_global();
+int __connman_storage_save_global();
+
 int __connman_storage_init_device();
 int __connman_storage_load_device(struct connman_device *device);
 int __connman_storage_save_device(struct connman_device *device);
-int __connman_storage_init_network(struct connman_device *device);
-int __connman_storage_load_network(struct connman_network *network);
-int __connman_storage_save_network(struct connman_network *network);
 int __connman_storage_init_service();
 int __connman_storage_load_service(struct connman_service *service);
 int __connman_storage_save_service(struct connman_service *service);
@@ -167,6 +174,8 @@ const char *__connman_element_get_network_path(struct connman_element *element);
 
 struct connman_device *__connman_element_find_device(enum connman_device_type type);
 int __connman_element_request_scan(enum connman_device_type type);
+int __connman_element_enable_technology(enum connman_device_type type);
+int __connman_element_disable_technology(enum connman_device_type type);
 
 const char *__connman_element_type2string(enum connman_element_type type);
 
@@ -201,13 +210,20 @@ char *__connman_udev_get_mbm_devnode(const char *ifname);
 int __connman_device_init(void);
 void __connman_device_cleanup(void);
 
+enum connman_service_type __connman_device_get_service_type(struct connman_device *device);
+
 void __connman_device_increase_connections(struct connman_device *device);
 void __connman_device_decrease_connections(struct connman_device *device);
 
 void __connman_device_set_network(struct connman_device *device,
 					struct connman_network *network);
+void __connman_device_cleanup_networks(struct connman_device *device);
 
 int __connman_device_scan(struct connman_device *device);
+int __connman_device_enable(struct connman_device *device);
+int __connman_device_enable_persistent(struct connman_device *device);
+int __connman_device_disable(struct connman_device *device);
+int __connman_device_disable_persistent(struct connman_device *device);
 int __connman_device_connect(struct connman_device *device);
 int __connman_device_disconnect(struct connman_device *device);
 
@@ -248,14 +264,17 @@ int __connman_service_init(void);
 void __connman_service_cleanup(void);
 
 void __connman_service_list(DBusMessageIter *iter);
+const char *__connman_service_default(void);
 
 void __connman_service_put(struct connman_service *service);
 
 struct connman_service *__connman_service_lookup_from_device(struct connman_device *device);
 struct connman_service *__connman_service_create_from_device(struct connman_device *device);
+void __connman_service_remove_from_device(struct connman_device *device);
 
 struct connman_service *__connman_service_lookup_from_network(struct connman_network *network);
 struct connman_service *__connman_service_create_from_network(struct connman_network *network);
+void __connman_service_remove_from_network(struct connman_network *network);
 
 unsigned int __connman_service_get_order(struct connman_service *service);
 
@@ -277,13 +296,17 @@ void __connman_service_auto_connect(void);
 int __connman_notifier_init(void);
 void __connman_notifier_cleanup(void);
 
-void __connman_notifier_device_type_list(gboolean powered,
-						DBusMessageIter *iter);
-void __connman_notifier_device_type_register(enum connman_device_type type);
-void __connman_notifier_device_type_unregister(enum connman_device_type type);
-void __connman_notifier_device_type_increase(enum connman_device_type type);
-void __connman_notifier_device_type_decrease(enum connman_device_type type);
-void __connman_notifier_offline_mode(connman_bool_t enabled);
+void __connman_notifier_list_registered(DBusMessageIter *iter);
+void __connman_notifier_list_enabled(DBusMessageIter *iter);
+void __connman_notifier_list_connected(DBusMessageIter *iter);
+
+void __connman_notifier_register(enum connman_service_type type);
+void __connman_notifier_unregister(enum connman_service_type type);
+void __connman_notifier_enable(enum connman_service_type type);
+void __connman_notifier_disable(enum connman_service_type type);
+void __connman_notifier_connect(enum connman_service_type type);
+void __connman_notifier_disconnect(enum connman_service_type type);
+void __connman_notifier_offlinemode(connman_bool_t enabled);
 
 #include <connman/rtnl.h>
 
