@@ -37,7 +37,9 @@ struct connman_device {
 	enum connman_device_type type;
 	enum connman_device_mode mode;
 	connman_bool_t secondary;
+	connman_bool_t offlinemode;
 	connman_bool_t powered;
+	connman_bool_t powered_persistent;
 	connman_bool_t carrier;
 	connman_bool_t scanning;
 	connman_bool_t disconnected;
@@ -46,6 +48,7 @@ struct connman_device {
 	char *node;
 	char *address;
 	char *interface;
+	char *control;
 	char *ident;
 	unsigned int connections;
 	guint scan_timeout;
@@ -81,6 +84,20 @@ static gboolean device_scan_trigger(gpointer user_data)
 	return TRUE;
 }
 
+static void reset_scan_trigger(struct connman_device *device)
+{
+	if (device->scan_timeout > 0) {
+		g_source_remove(device->scan_timeout);
+		device->scan_timeout = 0;
+	}
+
+	if (device->scan_interval > 0) {
+		guint interval = device->scan_interval;
+		device->scan_timeout = g_timeout_add_seconds(interval,
+					device_scan_trigger, device);
+	}
+}
+
 static const char *type2description(enum connman_device_type type)
 {
 	switch (type) {
@@ -97,6 +114,7 @@ static const char *type2description(enum connman_device_type type)
 		return "Bluetooth";
 	case CONNMAN_DEVICE_TYPE_GPS:
 		return "GPS";
+	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
@@ -123,6 +141,7 @@ static const char *type2string(enum connman_device_type type)
 		return "bluetooth";
 	case CONNMAN_DEVICE_TYPE_GPS:
 		return "gps";
+	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
@@ -133,6 +152,34 @@ static const char *type2string(enum connman_device_type type)
 	return NULL;
 }
 
+enum connman_service_type __connman_device_get_service_type(struct connman_device *device)
+{
+	enum connman_device_type type = connman_device_get_type(device);
+
+	switch (type) {
+	case CONNMAN_DEVICE_TYPE_UNKNOWN:
+	case CONNMAN_DEVICE_TYPE_VENDOR:
+	case CONNMAN_DEVICE_TYPE_GPS:
+	case CONNMAN_DEVICE_TYPE_NOZOMI:
+	case CONNMAN_DEVICE_TYPE_HUAWEI:
+	case CONNMAN_DEVICE_TYPE_NOVATEL:
+		break;
+	case CONNMAN_DEVICE_TYPE_ETHERNET:
+		return CONNMAN_SERVICE_TYPE_ETHERNET;
+	case CONNMAN_DEVICE_TYPE_WIFI:
+		return CONNMAN_SERVICE_TYPE_WIFI;
+	case CONNMAN_DEVICE_TYPE_WIMAX:
+		return CONNMAN_SERVICE_TYPE_WIMAX;
+	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
+		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	case CONNMAN_DEVICE_TYPE_MBM:
+	case CONNMAN_DEVICE_TYPE_HSO:
+		return CONNMAN_SERVICE_TYPE_CELLULAR;
+	}
+
+	return CONNMAN_SERVICE_TYPE_UNKNOWN;
+}
+
 static int set_connected(struct connman_device *device,
 						connman_bool_t connected)
 {
@@ -172,9 +219,27 @@ static int set_connected(struct connman_device *device,
 					CONNMAN_SERVICE_STATE_CONFIGURATION);
 		}
 	} else {
+		struct connman_service *service;
+
 		connman_element_unregister_children(&device->element);
 
 		device->disconnected = TRUE;
+
+		service = __connman_service_lookup_from_device(device);
+		__connman_service_indicate_state(service,
+					CONNMAN_SERVICE_STATE_IDLE);
+	}
+
+	if (connected == TRUE) {
+		enum connman_service_type type;
+
+		type = __connman_device_get_service_type(device);
+		__connman_notifier_connect(type);
+	} else {
+		enum connman_service_type type;
+
+		type = __connman_device_get_service_type(device);
+		__connman_notifier_disconnect(type);
 	}
 
 	return 0;
@@ -184,26 +249,64 @@ static int set_carrier(struct connman_device *device, connman_bool_t carrier)
 {
 	struct connman_service *service;
 
+	if (carrier == TRUE)
+		__connman_profile_add_device(device);
+	else
+		__connman_profile_remove_device(device);
+
 	service = __connman_service_lookup_from_device(device);
 	__connman_service_set_carrier(service, carrier);
 
 	return set_connected(device, carrier);
 }
 
+static int powered_changed(struct connman_device *device)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "Powered";
+
+	signal = dbus_message_new_signal(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return -ENOMEM;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
+							&device->powered);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+
+	return 0;
+}
+
 static int set_powered(struct connman_device *device, connman_bool_t powered)
 {
 	struct connman_device_driver *driver = device->driver;
+	enum connman_service_type type;
 	int err;
 
 	DBG("device %p powered %d", device, powered);
 
+	if (device->powered == powered)
+		return -EALREADY;
+
 	if (!driver)
 		return -EINVAL;
 
+	type = __connman_device_get_service_type(device);
+
 	if (powered == TRUE) {
 		if (driver->enable) {
 			err = driver->enable(device);
-			__connman_notifier_device_type_increase(device->type);
+			if (err == 0)
+				__connman_notifier_enable(type);
 		} else
 			err = -EINVAL;
 	} else {
@@ -213,11 +316,17 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 		if (driver->disable) {
 			err = driver->disable(device);
-			__connman_notifier_device_type_decrease(device->type);
+			if (err == 0)
+				__connman_notifier_disable(type);
 		} else
 			err = -EINVAL;
 	}
 
+	if (err == 0) {
+		device->powered = powered;
+		powered_changed(device);
+	}
+
 	return err;
 }
 
@@ -377,6 +486,10 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		dbus_message_iter_get_basic(&value, &powered);
 
+		device->powered_persistent = powered;
+
+		__connman_storage_save_device(device);
+
 		if (device->powered == powered)
 			return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 
@@ -412,17 +525,12 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		dbus_message_iter_get_basic(&value, &interval);
 
-		device->scan_interval = interval;
+		if (device->scan_interval != interval) {
+			device->scan_interval = interval;
 
-		if (device->scan_timeout > 0) {
-			g_source_remove(device->scan_timeout);
-			device->scan_timeout = 0;
-		}
+			__connman_storage_save_device(device);
 
-		if (device->scan_interval > 0) {
-			guint interval = device->scan_interval;
-			device->scan_timeout = g_timeout_add_seconds(interval,
-						device_scan_trigger, device);
+			reset_scan_trigger(device);
 		}
 	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
 		int err;
@@ -443,137 +551,9 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else
 		return __connman_error_invalid_property(msg);
 
-	__connman_storage_save_device(device);
-
 	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
 }
 
-static char *build_group(const unsigned char *ssid, unsigned int ssid_len,
-					const char *mode, const char *security)
-{
-	GString *str;
-	unsigned int i;
-
-	str = g_string_sized_new((ssid_len * 2) + 24);
-	if (str == NULL)
-		return NULL;
-
-	if (ssid_len > 0 && ssid[0] != '\0') {
-		for (i = 0; i < ssid_len; i++)
-			g_string_append_printf(str, "%02x", ssid[i]);
-	}
-
-	g_string_append_printf(str, "_%s_%s", mode, security);
-
-	return g_string_free(str, FALSE);
-}
-
-static DBusMessage *join_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	struct connman_device *device = data;
-	struct connman_network *network;
-	enum connman_network_type type;
-	unsigned int ssid_size;
-	const char *group, *mode, *security;
-	const void *ssid;
-	DBusMessageIter iter, array;
-	int err, index;
-
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	if (!device->driver || !device->driver->join)
-		return __connman_error_not_supported(msg);
-
-	dbus_message_iter_init(msg, &iter);
-	dbus_message_iter_recurse(&iter, &array);
-
-	switch (device->type) {
-	case CONNMAN_DEVICE_TYPE_WIFI:
-		type = CONNMAN_NETWORK_TYPE_WIFI;
-		break;
-	default:
-		return __connman_error_not_supported(msg);
-	}
-
-	network = connman_network_create("00_00_00_00_00_00", type);
-	if (network == NULL)
-		return __connman_error_failed(msg, ENOMEM);
-
-	while (dbus_message_iter_get_arg_type(&array) == DBUS_TYPE_DICT_ENTRY) {
-		DBusMessageIter entry, value;
-		const char *key, *str;
-
-		dbus_message_iter_recurse(&array, &entry);
-		dbus_message_iter_get_basic(&entry, &key);
-
-		dbus_message_iter_next(&entry);
-		dbus_message_iter_recurse(&entry, &value);
-
-		switch (dbus_message_iter_get_arg_type(&value)) {
-		case DBUS_TYPE_STRING:
-			dbus_message_iter_get_basic(&value, &str);
-			if (g_str_equal(key, "WiFi.SSID") == TRUE)
-				connman_network_set_blob(network, key,
-							str, strlen(str));
-			else
-				connman_network_set_string(network, key, str);
-			break;
-		}
-
-		dbus_message_iter_next(&array);
-	}
-
-	ssid = connman_network_get_blob(network, "WiFi.SSID", &ssid_size);
-	mode = connman_network_get_string(network, "WiFi.Mode");
-	security = connman_network_get_string(network, "WiFi.Security");
-	group = build_group(ssid, ssid_size, mode, security);
-
-	connman_network_set_group(network, group);
-
-	index = connman_device_get_index(device);
-	connman_network_set_index(network, index);
-
-	connman_network_set_protocol(network, CONNMAN_NETWORK_PROTOCOL_IP);
-
-	err = device->driver->join(device, network);
-
-	connman_network_unref(network);
-
-	if (err < 0)
-		return __connman_error_failed(msg, -err);
-
-	return g_dbus_create_reply(msg, DBUS_TYPE_INVALID);
-}
-
-static DBusMessage *create_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	return __connman_error_invalid_arguments(msg);
-}
-
-static DBusMessage *remove_network(DBusConnection *conn,
-					DBusMessage *msg, void *data)
-{
-	DBG("conn %p", conn);
-
-	if (__connman_security_check_privilege(msg,
-					CONNMAN_SECURITY_PRIVILEGE_MODIFY) < 0)
-		return __connman_error_permission_denied(msg);
-
-	return __connman_error_invalid_arguments(msg);
-}
-
 static DBusMessage *propose_scan(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
@@ -591,13 +571,7 @@ static DBusMessage *propose_scan(DBusConnection *conn,
 		break;
 	}
 
-	if (!device->driver || !device->driver->scan)
-		return __connman_error_not_supported(msg);
-
-	if (device->powered == FALSE)
-		return __connman_error_failed(msg, EINVAL);
-
-	err = device->driver->scan(device);
+	err = __connman_device_scan(device);
 	if (err < 0)
 		return __connman_error_failed(msg, -err);
 
@@ -608,9 +582,6 @@ static GDBusMethodTable device_methods[] = {
 	{ "GetProperties", "",      "a{sv}", get_properties },
 	{ "SetProperty",   "sv",    "",      set_property,
 						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "JoinNetwork",   "a{sv}", "",      join_network   },
-	{ "CreateNetwork", "a{sv}", "o",     create_network },
-	{ "RemoveNetwork", "o",     "",      remove_network },
 	{ "ProposeScan",   "",      "",      propose_scan   },
 	{ },
 };
@@ -691,34 +662,6 @@ static void unregister_interface(struct connman_element *element)
 						CONNMAN_DEVICE_INTERFACE);
 }
 
-static void device_enable(struct connman_device *device)
-{
-	DBG("device %p", device);
-
-	if (device->powered == TRUE)
-		return;
-
-	if (device->driver->enable) {
-		device->driver->enable(device);
-		__connman_notifier_device_type_increase(device->type);
-	}
-}
-
-static void device_disable(struct connman_device *device)
-{
-	DBG("device %p", device);
-
-	if (device->powered == FALSE)
-		return;
-
-	g_hash_table_remove_all(device->networks);
-
-	if (device->driver->disable) {
-		device->driver->disable(device);
-		__connman_notifier_device_type_decrease(device->type);
-	}
-}
-
 static int setup_device(struct connman_device *device)
 {
 	int err;
@@ -739,12 +682,14 @@ static int setup_device(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		break;
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		if (device->secondary == FALSE)
+		if (device->carrier == TRUE && device->secondary == FALSE)
 			__connman_profile_add_device(device);
 		break;
 	}
 
-	device_enable(device);
+	if (device->offlinemode == FALSE &&
+				device->powered_persistent == TRUE)
+		__connman_device_enable(device);
 
 	return 0;
 }
@@ -776,7 +721,7 @@ static void remove_device(struct connman_device *device)
 {
 	DBG("device %p", device);
 
-	device_disable(device);
+	__connman_device_disable(device);
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
@@ -896,6 +841,7 @@ static void device_destruct(struct connman_element *element)
 	g_free(device->node);
 	g_free(device->name);
 	g_free(device->address);
+	g_free(device->control);
 	g_free(device->interface);
 
 	connman_ipconfig_unref(device->ipconfig);
@@ -949,6 +895,8 @@ struct connman_device *connman_device_create(const char *node,
 	device->mode      = CONNMAN_DEVICE_MODE_UNKNOWN;
 	device->secondary = FALSE;
 
+	device->powered_persistent = TRUE;
+
 	switch (type) {
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
@@ -967,6 +915,7 @@ struct connman_device *connman_device_create(const char *node,
 	case CONNMAN_DEVICE_TYPE_GPS:
 		device->scan_interval = 0;
 		break;
+	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 	case CONNMAN_DEVICE_TYPE_NOZOMI:
 	case CONNMAN_DEVICE_TYPE_HUAWEI:
@@ -1077,11 +1026,12 @@ int connman_device_get_index(struct connman_device *device)
  * connman_device_set_interface:
  * @device: device structure
  * @interface: interface name
+ * @control: control interface
  *
  * Set interface name of device
  */
 void connman_device_set_interface(struct connman_device *device,
-							const char *interface)
+				const char *interface, const char *control)
 {
 	g_free(device->element.devname);
 	device->element.devname = g_strdup(interface);
@@ -1089,6 +1039,9 @@ void connman_device_set_interface(struct connman_device *device,
 	g_free(device->interface);
 	device->interface = g_strdup(interface);
 
+	g_free(device->control);
+	device->control = g_strdup(control);
+
 	if (device->name == NULL) {
 		const char *str = type2description(device->type);
 		if (str != NULL && device->interface != NULL)
@@ -1097,15 +1050,9 @@ void connman_device_set_interface(struct connman_device *device,
 	}
 }
 
-/**
- * connman_device_get_interface:
- * @device: device structure
- *
- * Get interface name of device
- */
-const char *connman_device_get_interface(struct connman_device *device)
+const char *connman_device_get_control(struct connman_device *device)
 {
-	return device->interface;
+	return device->control;
 }
 
 /**
@@ -1159,7 +1106,7 @@ enum connman_device_mode connman_device_get_mode(struct connman_device *device)
  * Change secondary value of device
  */
 void connman_device_set_secondary(struct connman_device *device,
-                                                connman_bool_t secondary)
+						connman_bool_t secondary)
 {
 	device->secondary = secondary;
 }
@@ -1185,9 +1132,7 @@ connman_bool_t connman_device_get_secondary(struct connman_device *device)
 int connman_device_set_powered(struct connman_device *device,
 						connman_bool_t powered)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Powered";
+	enum connman_service_type type;
 
 	DBG("driver %p powered %d", device, powered);
 
@@ -1209,38 +1154,22 @@ int connman_device_set_powered(struct connman_device *device,
 
 	device->powered = powered;
 
+	type = __connman_device_get_service_type(device);
+
+	if (device->powered == TRUE)
+		__connman_notifier_enable(type);
+	else
+		__connman_notifier_disable(type);
+
 	if (device->registered == FALSE)
 		return 0;
 
-	signal = dbus_message_new_signal(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &powered);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	powered_changed(device);
 
 	if (powered == FALSE)
 		return 0;
 
-	if (device->scan_timeout > 0) {
-		g_source_remove(device->scan_timeout);
-		device->scan_timeout = 0;
-	}
-
-	if (device->scan_interval > 0) {
-		guint interval = device->scan_interval;
-		device->scan_timeout = g_timeout_add_seconds(interval,
-					device_scan_trigger, device);
-	}
+	reset_scan_trigger(device);
 
 	if (device->driver->scan)
 		device->driver->scan(device);
@@ -1277,6 +1206,91 @@ int connman_device_set_carrier(struct connman_device *device,
 	return set_carrier(device, device->carrier);
 }
 
+int __connman_device_scan(struct connman_device *device)
+{
+	if (!device->driver || !device->driver->scan)
+		return -EOPNOTSUPP;
+
+	if (device->powered == FALSE)
+		return -ENOLINK;
+
+	return device->driver->scan(device);
+}
+
+int __connman_device_enable(struct connman_device *device)
+{
+	enum connman_service_type type;
+	int err;
+
+	DBG("device %p", device);
+
+	if (!device->driver || !device->driver->enable)
+		return -EOPNOTSUPP;
+
+	if (device->powered == TRUE)
+		return -EALREADY;
+
+	err = device->driver->enable(device);
+	if (err < 0)
+		return err;
+
+	device->powered = TRUE;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_enable(type);
+
+	return 0;
+}
+
+int __connman_device_enable_persistent(struct connman_device *device)
+{
+	DBG("device %p", device);
+
+	device->powered_persistent = TRUE;
+
+	__connman_storage_save_device(device);
+
+	return __connman_device_enable(device);
+}
+
+int __connman_device_disable(struct connman_device *device)
+{
+	enum connman_service_type type;
+	int err;
+
+	DBG("device %p", device);
+
+	if (!device->driver || !device->driver->disable)
+		return -EOPNOTSUPP;
+
+	if (device->powered == FALSE)
+		return -ENOLINK;
+
+	g_hash_table_remove_all(device->networks);
+
+	err = device->driver->disable(device);
+	if (err < 0)
+		return err;
+
+	device->powered = FALSE;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_disable(type);
+
+	return 0;
+}
+
+int __connman_device_disable_persistent(struct connman_device *device)
+{
+	DBG("device %p", device);
+
+	device->powered_persistent = FALSE;
+
+	__connman_storage_save_device(device);
+
+	return __connman_device_disable(device);
+}
+
 int __connman_device_connect(struct connman_device *device)
 {
 	DBG("device %p", device);
@@ -1304,8 +1318,7 @@ int __connman_device_disconnect(struct connman_device *device)
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
 		struct connman_network *network = value;
 
-		if (connman_network_get_connected(network) == TRUE)
-			__connman_network_disconnect(network);
+		__connman_network_disconnect(network);
 	}
 
 	if (device->driver && device->driver->disconnect)
@@ -1314,68 +1327,6 @@ int __connman_device_disconnect(struct connman_device *device)
 	return 0;
 }
 
-static void connect_known_network(struct connman_device *device)
-{
-	struct connman_network *network = NULL;
-	GHashTableIter iter;
-	gpointer key, value;
-	const char *name;
-	unsigned int count = 0;
-
-	DBG("device %p", device);
-
-	g_hash_table_iter_init(&iter, device->networks);
-
-	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
-		connman_uint8_t old_strength, new_strength;
-
-		count++;
-
-		if (connman_network_get_available(value) == FALSE)
-			continue;
-
-		name = connman_network_get_string(value,
-						CONNMAN_PROPERTY_ID_NAME);
-		if (name != NULL && device->last_network != NULL) {
-			if (g_str_equal(name, device->last_network) == TRUE) {
-				network = value;
-				break;
-			}
-		}
-
-		if (network == NULL) {
-			network = value;
-			continue;
-		}
-
-		old_strength = connman_network_get_uint8(network,
-						CONNMAN_PROPERTY_ID_STRENGTH);
-		new_strength = connman_network_get_uint8(value,
-						CONNMAN_PROPERTY_ID_STRENGTH);
-
-		if (new_strength > old_strength)
-			network = value;
-	}
-
-	if (network != NULL) {
-		int err;
-
-		name = connman_network_get_string(value,
-						CONNMAN_PROPERTY_ID_NAME);
-		if (name != NULL) {
-			err = __connman_network_connect(network);
-			if (err == 0 || err == -EINPROGRESS)
-				return;
-		}
-	}
-
-	if (count > 0)
-		return;
-
-	if (device->driver && device->driver->scan)
-		device->driver->scan(device);
-}
-
 static void mark_network_unavailable(gpointer key, gpointer value,
 							gpointer user_data)
 {
@@ -1401,6 +1352,12 @@ static gboolean remove_unavailable_network(gpointer key, gpointer value,
 	return TRUE;
 }
 
+void __connman_device_cleanup_networks(struct connman_device *device)
+{
+	g_hash_table_foreach_remove(device->networks,
+					remove_unavailable_network, NULL);
+}
+
 /**
  * connman_device_set_scanning:
  * @device: device structure
@@ -1442,24 +1399,15 @@ int connman_device_set_scanning(struct connman_device *device,
 	g_dbus_send_message(connection, signal);
 
 	if (scanning == TRUE) {
-		if (device->scan_timeout > 0) {
-			g_source_remove(device->scan_timeout);
-			device->scan_timeout = 0;
-		}
-
-		if (device->scan_interval > 0) {
-			guint interval = device->scan_interval;
-			device->scan_timeout = g_timeout_add_seconds(interval,
-						device_scan_trigger, device);
-		}
+		reset_scan_trigger(device);
 
 		g_hash_table_foreach(device->networks,
 					mark_network_unavailable, NULL);
+
 		return 0;
 	}
 
-	g_hash_table_foreach_remove(device->networks,
-					remove_unavailable_network, NULL);
+	__connman_device_cleanup_networks(device);
 
 	if (device->connections > 0)
 		return 0;
@@ -1467,7 +1415,7 @@ int connman_device_set_scanning(struct connman_device *device,
 	if (device->disconnected == TRUE)
 		return 0;
 
-	connect_known_network(device);
+	__connman_service_auto_connect();
 
 	return 0;
 }
@@ -1501,6 +1449,17 @@ int connman_device_set_disconnected(struct connman_device *device,
 	return 0;
 }
 
+/**
+ * connman_device_get_disconnected:
+ * @device: device structure
+ *
+ * Get device disconnected state
+ */
+connman_bool_t connman_device_get_disconnected(struct connman_device *device)
+{
+	return device->disconnected;
+}
+
 /**
  * connman_device_set_connected:
  * @device: device structure
@@ -1588,11 +1547,16 @@ static void set_offlinemode(struct connman_element *element, gpointer user_data)
 	if (device == NULL)
 		return;
 
+	device->offlinemode = offlinemode;
+
 	powered = (offlinemode == TRUE) ? FALSE : TRUE;
 
 	if (device->powered == powered)
 		return;
 
+	if (device->powered_persistent == FALSE)
+		powered = FALSE;
+
 	set_powered(device, powered);
 }
 
@@ -1603,19 +1567,29 @@ int __connman_device_set_offlinemode(connman_bool_t offlinemode)
 	__connman_element_foreach(NULL, CONNMAN_ELEMENT_TYPE_DEVICE,
 			set_offlinemode, GUINT_TO_POINTER(offlinemode));
 
-	__connman_notifier_offline_mode(offlinemode);
+	__connman_notifier_offlinemode(offlinemode);
 
 	return 0;
 }
 
 void __connman_device_increase_connections(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	device->connections++;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_connect(type);
 }
 
 void __connman_device_decrease_connections(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	device->connections--;
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_disconnect(type);
 }
 
 /**
@@ -1644,8 +1618,6 @@ int connman_device_add_network(struct connman_device *device,
 
 	__connman_network_set_device(network, device);
 
-	__connman_storage_load_network(network);
-
 	err = connman_element_register((struct connman_element *) network,
 							&device->element);
 	if (err < 0) {
@@ -1725,17 +1697,14 @@ void __connman_device_set_network(struct connman_device *device,
  */
 int connman_device_register(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	__connman_storage_load_device(device);
 
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		break;
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		__connman_storage_init_network(device);
-		break;
-	}
+	device->offlinemode = __connman_manager_get_offlinemode();
+
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_register(type);
 
 	return connman_element_register(&device->element, NULL);
 }
@@ -1748,8 +1717,13 @@ int connman_device_register(struct connman_device *device)
  */
 void connman_device_unregister(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	__connman_storage_save_device(device);
 
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_unregister(type);
+
 	connman_element_unregister(&device->element);
 }
 
@@ -1845,38 +1819,27 @@ static struct connman_driver device_driver = {
 static int device_load(struct connman_device *device)
 {
 	GKeyFile *keyfile;
-	gchar *pathname, *identifier, *data = NULL;
-	gsize length;
+	GError *error = NULL;
+	gchar *identifier;
+	connman_bool_t powered;
 	int val;
 
 	DBG("device %p", device);
 
-	pathname = g_strdup_printf("%s/%s.conf", STORAGEDIR,
-					__connman_profile_active_ident());
-	if (pathname == NULL)
-		return -ENOMEM;
-
-	keyfile = g_key_file_new();
-
-	if (g_file_get_contents(pathname, &data, &length, NULL) == FALSE) {
-		g_free(pathname);
-		return -ENOENT;
-	}
-
-	g_free(pathname);
-
-	if (g_key_file_load_from_data(keyfile, data, length,
-							0, NULL) == FALSE) {
-		g_free(data);
-		return -EILSEQ;
-	}
-
-	g_free(data);
+	keyfile = __connman_storage_open();
+	if (keyfile == NULL)
+		return 0;
 
 	identifier = g_strdup_printf("device_%s", device->element.name);
 	if (identifier == NULL)
 		goto done;
 
+	powered = g_key_file_get_boolean(keyfile, identifier,
+						"Powered", &error);
+	if (error == NULL)
+		device->powered_persistent = powered;
+	g_clear_error(&error);
+
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
@@ -1884,51 +1847,39 @@ static int device_load(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		val = g_key_file_get_integer(keyfile, identifier,
-							"ScanInterval", NULL);
-		if (val > 0)
+						"ScanInterval", &error);
+		if (error == NULL && val > 0)
 			device->scan_interval = val;
+		g_clear_error(&error);
 		break;
 	}
 
 done:
-	g_key_file_free(keyfile);
-
 	g_free(identifier);
 
+	__connman_storage_close(keyfile, FALSE);
+
 	return 0;
 }
 
 static int device_save(struct connman_device *device)
 {
 	GKeyFile *keyfile;
-	gchar *pathname, *identifier = NULL, *data = NULL;
-	gsize length;
+	gchar *identifier;
 
 	DBG("device %p", device);
 
-	pathname = g_strdup_printf("%s/%s.conf", STORAGEDIR,
-					__connman_profile_active_ident());
-	if (pathname == NULL)
-		return -ENOMEM;
+	keyfile = __connman_storage_open();
+	if (keyfile == NULL)
+		return 0;
 
-	keyfile = g_key_file_new();
-
-	if (g_file_get_contents(pathname, &data, &length, NULL) == FALSE)
-		goto update;
-
-	if (length > 0) {
-		if (g_key_file_load_from_data(keyfile, data, length,
-							0, NULL) == FALSE)
-			goto done;
-	}
-
-	g_free(data);
-
-update:
 	identifier = g_strdup_printf("device_%s", device->element.name);
 	if (identifier == NULL)
 		goto done;
 
+	g_key_file_set_boolean(keyfile, identifier,
+					"Powered", device->powered_persistent);
+
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
 	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
@@ -1941,18 +1892,10 @@ update:
 		break;
 	}
 
-	data = g_key_file_to_data(keyfile, &length, NULL);
-
-	if (g_file_set_contents(pathname, data, length, NULL) == FALSE)
-		connman_error("Failed to store device information");
-
 done:
-	g_free(data);
-
-	g_key_file_free(keyfile);
-
 	g_free(identifier);
-	g_free(pathname);
+
+	__connman_storage_close(keyfile, TRUE);
 
 	return 0;
 }
