@@ -40,6 +40,8 @@ static GMainLoop *main_loop = NULL;
 
 static void sig_term(int sig)
 {
+	connman_info("Terminating");
+
 	g_main_loop_quit(main_loop);
 }
 
@@ -50,7 +52,7 @@ static void sig_debug(int sig)
 
 static void disconnect_callback(DBusConnection *conn, void *user_data)
 {
-	DBG("D-Bus disconnect");
+	connman_error("D-Bus disconnect");
 
 	g_main_loop_quit(main_loop);
 }
@@ -203,8 +205,6 @@ int main(int argc, char *argv[])
 	__connman_profile_init(conn);
 
 	__connman_resolver_init();
-	__connman_rtnl_init();
-	__connman_udev_init();
 
 	__connman_plugin_init(option_plugin, option_noplugin);
 
